@@ -15,9 +15,6 @@ import itertools
 import os
 import re
 
-import pkg_resources
-import six
-
 from oslo_config import cfg
 from oslo_log import log
 from stevedore import extension
@@ -28,15 +25,6 @@ from ceilometer.pipeline import sample as endpoint
 from ceilometer import sample as sample_util
 
 OPTS = [
-    cfg.StrOpt('meter_definitions_cfg_file',
-               deprecated_for_removal=True,
-               help="Configuration file for defining meter "
-                    "notifications. This option is deprecated "
-                    "and use meter_definitions_dirs to "
-                    "configure meter notification file. Meter "
-                    "definitions configuration file will be sought "
-                    "according to the parameter."
-               ),
     cfg.MultiStrOpt('meter_definitions_dirs',
                     default=["/etc/ceilometer/meters.d",
                              os.path.abspath(
@@ -70,7 +58,7 @@ class MeterDefinition(object):
                 _("Required fields %s not specified") % missing, self.cfg)
 
         self._event_type = self.cfg.get('event_type')
-        if isinstance(self._event_type, six.string_types):
+        if isinstance(self._event_type, str):
             self._event_type = [self._event_type]
         self._event_type = [re.compile(etype) for etype in self._event_type]
 
@@ -82,7 +70,7 @@ class MeterDefinition(object):
         self._fallback_user_id = declarative.Definition(
             'user_id', "ctxt.user_id|ctxt.user", plugin_manager)
         self._fallback_project_id = declarative.Definition(
-            'project_id', "ctxt.tenant_id|ctxt.tenant", plugin_manager)
+            'project_id', "ctxt.project_id|ctxt.tenant_id", plugin_manager)
         self._attributes = {}
         self._metadata_attributes = {}
         self._user_meta = None
@@ -103,7 +91,7 @@ class MeterDefinition(object):
 
         # List of fields we expected when multiple meter are in the payload
         self.lookup = self.cfg.get('lookup')
-        if isinstance(self.lookup, six.string_types):
+        if isinstance(self.lookup, str):
             self.lookup = [self.lookup]
 
     def match_type(self, meter_name):
@@ -200,11 +188,6 @@ class ProcessMeterNotifications(endpoint.SampleEndpoint):
             for filepath in sorted(glob.glob(os.path.join(dir, "*.yaml"))):
                 if filepath is not None:
                     mfs.append(filepath)
-        if self.conf.meter.meter_definitions_cfg_file is not None:
-            mfs.append(
-                pkg_resources.resource_filename(
-                    self.conf.meter.meter_definitions_cfg_file)
-            )
         for mf in mfs:
             meters_cfg = declarative.load_definitions(
                 self.conf, {}, mf)
@@ -219,7 +202,7 @@ class ProcessMeterNotifications(endpoint.SampleEndpoint):
                     md = MeterDefinition(meter_cfg, self.conf, plugin_manager)
                 except declarative.DefinitionException as e:
                     errmsg = "Error loading meter definition: %s"
-                    LOG.error(errmsg, six.text_type(e))
+                    LOG.error(errmsg, str(e))
                 else:
                     definitions[meter_cfg['name']] = md
         return definitions.values()
