@@ -21,10 +21,7 @@ import ceilometer.compute.discovery
 import ceilometer.compute.virt.inspector
 import ceilometer.compute.virt.libvirt.utils
 import ceilometer.compute.virt.vmware.inspector
-import ceilometer.compute.virt.xenapi.inspector
 import ceilometer.event.converter
-import ceilometer.hardware.discovery
-import ceilometer.hardware.pollsters.generic
 import ceilometer.image.discovery
 import ceilometer.ipmi.platform.intel_node_manager
 import ceilometer.ipmi.pollsters
@@ -86,17 +83,9 @@ def list_opts():
                 'left empty, per-deployment central agent and per-host '
                 'compute agent won\'t do workload '
                 'partitioning and will only function correctly if a '
-                'single instance of that service is running.'),
-            cfg.FloatOpt(
-                'check_watchers',
-                default=10.0,
-                help='Number of seconds between checks to see if group '
-                'membership has changed'),
+                'single instance of that service is running.')
         ]),
         ('event', ceilometer.event.converter.OPTS),
-        ('hardware', itertools.chain(
-            ceilometer.hardware.discovery.OPTS,
-            ceilometer.hardware.pollsters.generic.OPTS)),
         ('ipmi',
          itertools.chain(ceilometer.ipmi.platform.intel_node_manager.OPTS,
                          ceilometer.ipmi.pollsters.OPTS)),
@@ -118,7 +107,6 @@ def list_opts():
                          ceilometer.objectstore.swift.SERVICE_OPTS,
                          ceilometer.volume.discovery.SERVICE_OPTS,)),
         ('vmware', ceilometer.compute.virt.vmware.inspector.OPTS),
-        ('xenapi', ceilometer.compute.virt.xenapi.inspector.OPTS),
     ]
 
 
