@@ -17,6 +17,8 @@ from unittest import mock
 import fixtures
 import testtools
 
+from novaclient import exceptions
+
 try:
     import libvirt
 except ImportError:
@@ -128,6 +130,9 @@ class FakeManualInstanceConn(object):
     def listAllDomains(self):
         return [FakeManualInstanceDomain()]
 
+    def isAlive(self):
+        return False
+
 
 class TestDiscovery(base.BaseTestCase):
 
@@ -286,14 +291,39 @@ class TestDiscovery(base.BaseTestCase):
                          self.client.instance_get_all_by_host.call_args_list)
 
     @testtools.skipUnless(libvirt, "libvirt not available")
-    @mock.patch.object(utils, "libvirt")
-    @mock.patch.object(discovery, "libvirt")
-    def test_discovery_with_libvirt_error(self, libvirt, libvirt2):
+    @mock.patch.object(libvirt, "VIR_DOMAIN_METADATA_ELEMENT", 2)
+    @mock.patch.object(libvirt, "openReadOnly")
+    def test_discovery_with_libvirt_error(self, openReadOnly):
         self.CONF.set_override("instance_discovery_method",
                                "libvirt_metadata",
                                group="compute")
-        libvirt.VIR_DOMAIN_METADATA_ELEMENT = 2
-        libvirt2.openReadOnly.return_value = FakeManualInstanceConn()
+        openReadOnly.return_value = FakeManualInstanceConn()
         dsc = discovery.InstanceDiscovery(self.CONF)
         resources = dsc.discover(mock.MagicMock())
         self.assertEqual(0, len(resources))
+
+    def test_get_flavor_id(self):
+        self.CONF.set_override("instance_discovery_method",
+                               "libvirt_metadata",
+                               group="compute")
+        self.client.nova_client = mock.MagicMock()
+        self.client.nova_client.flavors = mock.MagicMock()
+        fake_flavor = mock.MagicMock()
+        fake_flavor.id = 'fake_id'
+
+        self.client.nova_client.flavors.find = mock.MagicMock(
+            return_value=fake_flavor)
+        dsc = discovery.InstanceDiscovery(self.CONF)
+
+        flavor_name = 'fake_name'
+
+        ret_flavor_id = dsc.get_flavor_id(flavor_name)
+        self.assertEqual('fake_id', ret_flavor_id)
+
+        # test raise NotFound exception
+        self.client.nova_client.flavors.find = mock.MagicMock(
+            side_effect=exceptions.NotFound(404))
+        dsc = discovery.InstanceDiscovery(self.CONF)
+
+        ret_flavor_id = dsc.get_flavor_id(flavor_name)
+        self.assertEqual(flavor_name, ret_flavor_id)
