@@ -10,71 +10,88 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-"""Tests for ceilometer.meter.notifications
-"""
+"""Tests for ceilometer.meter.notifications"""
 import copy
-import mock
-import os
-import six
-import yaml
+from unittest import mock
 
-from oslo_config import fixture as fixture_config
+import fixtures
 from oslo_utils import encodeutils
 from oslo_utils import fileutils
+import yaml
 
-import ceilometer
 from ceilometer import declarative
 from ceilometer.meter import notifications
 from ceilometer import service as ceilometer_service
 from ceilometer.tests import base as test
 
 NOTIFICATION = {
-    'event_type': u'test.create',
-    'timestamp': u'2015-06-1909: 19: 35.786893',
-    'payload': {u'user_id': u'e1d870e51c7340cb9d555b15cbfcaec2',
-                u'resource_id': u'bea70e51c7340cb9d555b15cbfcaec23',
-                u'timestamp': u'2015-06-19T09:19:35.785330',
-                u'created_at': u'2015-06-19T09:25:35.785330',
-                u'launched_at': u'2015-06-19T09:25:40.785330',
-                u'message_signature': u'fake_signature1',
-                u'resource_metadata': {u'foo': u'bar'},
-                u'source': u'30be1fc9a03c4e94ab05c403a8a377f2: openstack',
-                u'volume': 1.0,
-                u'project_id': u'30be1fc9a03c4e94ab05c403a8a377f2',
+    'event_type': 'test.create',
+    'metadata': {'timestamp': '2015-06-19T09:19:35.786893',
+                 'message_id': '939823de-c242-45a2-a399-083f4d6a8c3e'},
+    'payload': {'user_id': 'e1d870e51c7340cb9d555b15cbfcaec2',
+                'resource_id': 'bea70e51c7340cb9d555b15cbfcaec23',
+                'timestamp': '2015-06-19T09:19:35.785330',
+                'created_at': '2015-06-19T09:25:35.785330',
+                'launched_at': '2015-06-19T09:25:40.785330',
+                'message_signature': 'fake_signature1',
+                'resource_metadata': {'foo': 'bar'},
+                'source': '30be1fc9a03c4e94ab05c403a8a377f2: openstack',
+                'volume': 1.0,
+                'project_id': '30be1fc9a03c4e94ab05c403a8a377f2',
                 },
-    u'_context_tenant': u'30be1fc9a03c4e94ab05c403a8a377f2',
-    u'_context_request_id': u'req-da91b4bf-d2b5-43ae-8b66-c7752e72726d',
-    u'_context_user': u'e1d870e51c7340cb9d555b15cbfcaec2',
-    'message_id': u'939823de-c242-45a2-a399-083f4d6a8c3e',
+    'ctxt': {'tenant': '30be1fc9a03c4e94ab05c403a8a377f2',
+             'request_id': 'req-da91b4bf-d2b5-43ae-8b66-c7752e72726d',
+             'user': 'e1d870e51c7340cb9d555b15cbfcaec2'},
+    'publisher_id': "foo123"
+}
+
+USER_META = {
+    'event_type': 'test.create',
+    'metadata': {'timestamp': '2015-06-19T09:19:35.786893',
+                 'message_id': '939823de-c242-45a2-a399-083f4d6a8c3e'},
+    'payload': {'user_id': 'e1d870e51c7340cb9d555b15cbfcaec2',
+                'resource_id': 'bea70e51c7340cb9d555b15cbfcaec23',
+                'timestamp': '2015-06-19T09:19:35.785330',
+                'created_at': '2015-06-19T09:25:35.785330',
+                'launched_at': '2015-06-19T09:25:40.785330',
+                'message_signature': 'fake_signature1',
+                'resource_metadata': {'foo': 'bar'},
+                'source': '30be1fc9a03c4e94ab05c403a8a377f2: openstack',
+                'volume': 1.0,
+                'project_id': '30be1fc9a03c4e94ab05c403a8a377f2',
+                'metadata': {'metering.xyz': 'abc', 'ignore': 'this'},
+                },
+    'ctxt': {'tenant': '30be1fc9a03c4e94ab05c403a8a377f2',
+             'request_id': 'req-da91b4bf-d2b5-43ae-8b66-c7752e72726d',
+             'user': 'e1d870e51c7340cb9d555b15cbfcaec2'},
     'publisher_id': "foo123"
 }
 
 MIDDLEWARE_EVENT = {
-    u'_context_request_id': u'req-a8bfa89b-d28b-4b95-9e4b-7d7875275650',
-    u'_context_quota_class': None,
-    u'event_type': u'objectstore.http.request',
-    u'_context_service_catalog': [],
-    u'_context_auth_token': None,
-    u'_context_user_id': None,
-    u'priority': u'INFO',
-    u'_context_is_admin': True,
-    u'_context_user': None,
-    u'publisher_id': u'ceilometermiddleware',
-    u'message_id': u'6eccedba-120e-4db8-9735-2ad5f061e5ee',
-    u'_context_remote_address': None,
-    u'_context_roles': [],
-    u'timestamp': u'2013-07-29 06:51:34.474815',
-    u'_context_timestamp': u'2013-07-29T06:51:34.348091',
-    u'_unique_id': u'0ee26117077648e18d88ac76e28a72e2',
-    u'_context_project_name': None,
-    u'_context_read_deleted': u'no',
-    u'_context_tenant': None,
-    u'_context_instance_lock_checked': False,
-    u'_context_project_id': None,
-    u'_context_user_name': None,
-    u'payload': {
+    'ctxt': {'request_id': 'req-a8bfa89b-d28b-4b95-9e4b-7d7875275650',
+             'quota_class': None,
+             'service_catalog': [],
+             'auth_token': None,
+             'user_id': None,
+             'is_admin': True,
+             'user': None,
+             'remote_address': None,
+             'roles': [],
+             'timestamp': '2013-07-29T06:51:34.348091',
+             'project_name': None,
+             'read_deleted': 'no',
+             'tenant': None,
+             'instance_lock_checked': False,
+             'project_id': None,
+             'user_name': None},
+    'event_type': 'objectstore.http.request',
+    'publisher_id': 'ceilometermiddleware',
+    'metadata': {'message_id': '6eccedba-120e-4db8-9735-2ad5f061e5ee',
+                  'timestamp': '2013-07-29T06:51:34.474815+00:00',
+                  '_unique_id': '0ee26117077648e18d88ac76e28a72e2'},
+    'payload': {
         'typeURI': 'http: //schemas.dmtf.org/cloud/audit/1.0/event',
-        'eventTime': '2015-01-30T16: 38: 43.233621',
+        'eventTime': '2013-07-29T06:51:34.474815+00:00',
         'target': {
             'action': 'get',
             'typeURI': 'service/storage/object',
@@ -120,101 +137,99 @@ MIDDLEWARE_EVENT = {
 }
 
 FULL_MULTI_MSG = {
-    u'_context_domain': None,
-    u'_context_request_id': u'req-da91b4bf-d2b5-43ae-8b66-c7752e72726d',
-    'event_type': u'full.sample',
-    'timestamp': u'2015-06-1909: 19: 35.786893',
-    u'_context_auth_token': None,
-    u'_context_read_only': False,
+    'event_type': 'full.sample',
     'payload': [{
-                u'counter_name': u'instance1',
-                u'user_id': u'user1',
-                u'resource_id': u'res1',
-                u'counter_unit': u'ns',
-                u'counter_volume': 28.0,
-                u'project_id': u'proj1',
-                u'counter_type': u'gauge'
+                'counter_name': 'instance1',
+                'user_id': 'user1',
+                'resource_id': 'res1',
+                'counter_unit': 'ns',
+                'counter_volume': 28.0,
+                'project_id': 'proj1',
+                'counter_type': 'gauge'
                 },
                 {
-                u'counter_name': u'instance2',
-                u'user_id': u'user2',
-                u'resource_id': u'res2',
-                u'counter_unit': u'%',
-                u'counter_volume': 1.0,
-                u'project_id': u'proj2',
-                u'counter_type': u'delta'
+                'counter_name': 'instance2',
+                'user_id': 'user2',
+                'resource_id': 'res2',
+                'counter_unit': '%',
+                'counter_volume': 1.0,
+                'project_id': 'proj2',
+                'counter_type': 'delta'
                 }],
-    u'_context_resource_uuid': None,
-    u'_context_user_identity': u'fake_user_identity---',
-    u'_context_show_deleted': False,
-    u'_context_tenant': u'30be1fc9a03c4e94ab05c403a8a377f2',
-    'priority': 'info',
-    u'_context_is_admin': True,
-    u'_context_project_domain': None,
-    u'_context_user': u'e1d870e51c7340cb9d555b15cbfcaec2',
-    u'_context_user_domain': None,
-    'publisher_id': u'ceilometer.api',
-    'message_id': u'939823de-c242-45a2-a399-083f4d6a8c3e'
+    'ctxt': {'domain': None,
+             'request_id': 'req-da91b4bf-d2b5-43ae-8b66-c7752e72726d',
+             'auth_token': None,
+             'read_only': False,
+             'resource_uuid': None,
+             'user_identity': 'fake_user_identity---',
+             'show_deleted': False,
+             'tenant': '30be1fc9a03c4e94ab05c403a8a377f2',
+             'is_admin': True,
+             'project_domain': None,
+             'user': 'e1d870e51c7340cb9d555b15cbfcaec2',
+             'user_domain': None},
+    'publisher_id': 'ceilometer.api',
+    'metadata': {'message_id': '939823de-c242-45a2-a399-083f4d6a8c3e',
+                 'timestamp': '2015-06-19T09:19:35.786893'},
 }
 
 METRICS_UPDATE = {
-    u'_context_request_id': u'req-a8bfa89b-d28b-4b95-9e4b-7d7875275650',
-    u'_context_quota_class': None,
-    u'event_type': u'compute.metrics.update',
-    u'_context_service_catalog': [],
-    u'_context_auth_token': None,
-    u'_context_user_id': None,
-    u'payload': {
-        u'metrics': [
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+    'event_type': 'compute.metrics.update',
+    'payload': {
+        'metrics': [
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.frequency', 'value': 1600,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.user.time', 'value': 17421440000000,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.kernel.time', 'value': 7852600000000,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.idle.time', 'value': 1307374400000000,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.iowait.time', 'value': 11697470000000,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.user.percent', 'value': 0.012959045637294348,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.kernel.percent', 'value': 0.005841204961898534,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.idle.percent', 'value': 0.9724985141658965,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.iowait.percent', 'value': 0.008701235234910634,
              'source': 'libvirt.LibvirtDriver'},
-            {'timestamp': u'2013-07-29T06:51:34.472416',
+            {'timestamp': '2013-07-29T06:51:34.472416',
              'name': 'cpu.percent', 'value': 0.027501485834103515,
              'source': 'libvirt.LibvirtDriver'}],
-        u'nodename': u'tianst.sh.intel.com',
-        u'host': u'tianst',
-        u'host_id': u'10.0.1.1'},
-    u'priority': u'INFO',
-    u'_context_is_admin': True,
-    u'_context_user': None,
-    u'publisher_id': u'compute.tianst.sh.intel.com',
-    u'message_id': u'6eccedba-120e-4db8-9735-2ad5f061e5ee',
-    u'_context_remote_address': None,
-    u'_context_roles': [],
-    u'timestamp': u'2013-07-29 06:51:34.474815',
-    u'_context_timestamp': u'2013-07-29T06:51:34.348091',
-    u'_unique_id': u'0ee26117077648e18d88ac76e28a72e2',
-    u'_context_project_name': None,
-    u'_context_read_deleted': u'no',
-    u'_context_tenant': None,
-    u'_context_instance_lock_checked': False,
-    u'_context_project_id': None,
-    u'_context_user_name': None
+        'nodename': 'tianst.sh.intel.com',
+        'host': 'tianst',
+        'host_id': '10.0.1.1'},
+    'publisher_id': 'compute.tianst.sh.intel.com',
+    'metadata': {'message_id': '6eccedba-120e-4db8-9735-2ad5f061e5ee',
+                 'timestamp': '2013-07-29 06:51:34.474815',
+                 '_unique_id': '0ee26117077648e18d88ac76e28a72e2'},
+    'ctxt': {'request_id': 'req-a8bfa89b-d28b-4b95-9e4b-7d7875275650',
+             'quota_class': None,
+             'service_catalog': [],
+             'auth_token': None,
+             'user_id': None,
+             'is_admin': True,
+             'user': None,
+             'remote_address': None,
+             'roles': [],
+             'timestamp': '2013-07-29T06:51:34.348091',
+             'project_name': None,
+             'read_deleted': 'no',
+             'tenant': None,
+             'instance_lock_checked': False,
+             'project_id': None,
+             'user_name': None}
 }
 
 
@@ -228,7 +243,7 @@ class TestMeterDefinition(test.BaseTestCase):
                    volume="$.payload.volume",
                    resource_id="$.payload.resource_id",
                    project_id="$.payload.project_id")
-        handler = notifications.MeterDefinition(cfg, mock.Mock())
+        handler = notifications.MeterDefinition(cfg, mock.Mock(), mock.Mock())
         self.assertTrue(handler.match_type("test.create"))
         sample = list(handler.to_samples(NOTIFICATION))[0]
         self.assertEqual(1.0, sample["volume"])
@@ -240,7 +255,7 @@ class TestMeterDefinition(test.BaseTestCase):
     def test_config_required_missing_fields(self):
         cfg = dict()
         try:
-            notifications.MeterDefinition(cfg, mock.Mock())
+            notifications.MeterDefinition(cfg, mock.Mock(), mock.Mock())
         except declarative.DefinitionException as e:
             self.assertIn("Required fields ['name', 'type', 'event_type',"
                           " 'unit', 'volume', 'resource_id']"
@@ -252,7 +267,7 @@ class TestMeterDefinition(test.BaseTestCase):
                    unit="foo", volume="bar",
                    resource_id="bea70e51c7340cb9d555b15cbfcaec23")
         try:
-            notifications.MeterDefinition(cfg, mock.Mock())
+            notifications.MeterDefinition(cfg, mock.Mock(), mock.Mock())
         except declarative.DefinitionException as e:
             self.assertIn("Invalid type foo specified",
                           encodeutils.exception_to_unicode(e))
@@ -262,31 +277,24 @@ class TestMeterProcessing(test.BaseTestCase):
 
     def setUp(self):
         super(TestMeterProcessing, self).setUp()
-        self.CONF = self.useFixture(fixture_config.Config()).conf
-        ceilometer_service.prepare_service(argv=[], config_files=[])
-        self.handler = notifications.ProcessMeterNotifications(mock.Mock())
+        self.CONF = ceilometer_service.prepare_service([], [])
+        self.path = self.useFixture(fixtures.TempDir()).path
+        self.handler = notifications.ProcessMeterNotifications(
+            self.CONF, mock.Mock())
 
-    def test_fallback_meter_path(self):
-        self.CONF.set_override('meter_definitions_cfg_file',
-                               '/not/existing/path', group='meter')
-        with mock.patch('ceilometer.declarative.open',
-                        mock.mock_open(read_data='---\nmetric: []'),
-                        create=True) as mock_open:
-            self.handler._load_definitions()
-            if six.PY3:
-                path = os.path.dirname(ceilometer.__file__)
-            else:
-                path = "ceilometer"
-            mock_open.assert_called_with(path + "/meter/data/meters.yaml")
-
-    def _load_meter_def_file(self, cfg):
-        if six.PY3:
+    def _load_meter_def_file(self, cfgs=None):
+        self.CONF.set_override('meter_definitions_dirs',
+                               [self.path], group='meter')
+        cfgs = cfgs or []
+        if not isinstance(cfgs, list):
+            cfgs = [cfgs]
+        meter_cfg_files = list()
+        for cfg in cfgs:
             cfg = cfg.encode('utf-8')
-        meter_cfg_file = fileutils.write_to_tempfile(content=cfg,
-                                                     prefix="meters",
-                                                     suffix="yaml")
-        self.CONF.set_override('meter_definitions_cfg_file',
-                               meter_cfg_file, group='meter')
+            meter_cfg_files.append(fileutils.write_to_tempfile(content=cfg,
+                                                               path=self.path,
+                                                               prefix="meters",
+                                                               suffix=".yaml"))
         self.handler.definitions = self.handler._load_definitions()
 
     @mock.patch('ceilometer.meter.notifications.LOG')
@@ -326,7 +334,7 @@ class TestMeterProcessing(test.BaseTestCase):
                              resource_id="$.payload.resource_id",
                              project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(NOTIFICATION))
+        c = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         self.assertEqual('test1', s1['name'])
@@ -351,7 +359,7 @@ class TestMeterProcessing(test.BaseTestCase):
                              resource_id="$.payload.resource_id",
                              project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
-        data = list(self.handler.process_notification(NOTIFICATION))
+        data = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(2, len(data))
         expected_names = ['test1', 'test2']
         for s in data:
@@ -367,7 +375,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         resource_id="$.payload.resource_id",
                         project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(NOTIFICATION))
+        c = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(0, len(c))
 
     def test_regex_match_meter(self):
@@ -380,7 +388,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         resource_id="$.payload.resource_id",
                         project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(NOTIFICATION))
+        c = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(1, len(c))
 
     def test_default_timestamp(self):
@@ -396,10 +404,11 @@ class TestMeterProcessing(test.BaseTestCase):
                         project_id="$.payload.initiator.project_id",
                         multi="name")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(event))
+        c = list(self.handler.build_sample(event))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
-        self.assertEqual(MIDDLEWARE_EVENT['timestamp'], s1['timestamp'])
+        self.assertEqual(MIDDLEWARE_EVENT['metadata']['timestamp'],
+                         s1['timestamp'])
 
     def test_custom_timestamp(self):
         event = copy.deepcopy(MIDDLEWARE_EVENT)
@@ -415,7 +424,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         multi="name",
                         timestamp='$.payload.eventTime')]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(event))
+        c = list(self.handler.build_sample(event))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         self.assertEqual(MIDDLEWARE_EVENT['payload']['eventTime'],
@@ -433,11 +442,11 @@ class TestMeterProcessing(test.BaseTestCase):
                         timestamp="$.payload.metrics"
                                   "[?(@.name='cpu.frequency')].timestamp")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(METRICS_UPDATE))
+        c = list(self.handler.build_sample(METRICS_UPDATE))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         self.assertEqual('compute.node.cpu.frequency', s1['name'])
-        self.assertEqual("2013-07-29T06:51:34.472416", s1['timestamp'])
+        self.assertEqual("2013-07-29T06:51:34.472416+00:00", s1['timestamp'])
 
     def test_default_metadata(self):
         cfg = yaml.dump(
@@ -449,7 +458,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         resource_id="$.payload.resource_id",
                         project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(NOTIFICATION))
+        c = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         meta = NOTIFICATION['payload'].copy()
@@ -469,7 +478,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         resource_id="$.payload.resource_id",
                         project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(NOTIFICATION))
+        c = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         self.assertEqual(5.0, s1['volume'])
@@ -486,13 +495,48 @@ class TestMeterProcessing(test.BaseTestCase):
                         metadata={'proj': '$.payload.project_id',
                                   'dict': '$.payload.resource_metadata'})]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(NOTIFICATION))
+        c = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         meta = {'proj': s1['project_id'],
                 'dict': NOTIFICATION['payload']['resource_metadata']}
         self.assertEqual(meta, s1['resource_metadata'])
 
+    def test_user_meta(self):
+        cfg = yaml.dump(
+            {'metric': [dict(name="test1",
+                        event_type="test.*",
+                        type="delta",
+                        unit="B",
+                        volume="$.payload.volume",
+                        resource_id="$.payload.resource_id",
+                        project_id="$.payload.project_id",
+                        user_metadata="$.payload.metadata",)]})
+        self._load_meter_def_file(cfg)
+        c = list(self.handler.build_sample(USER_META))
+        self.assertEqual(1, len(c))
+        s1 = c[0].as_dict()
+        meta = {'user_metadata': {'xyz': 'abc'}}
+        self.assertEqual(meta, s1['resource_metadata'])
+
+    def test_user_meta_and_custom(self):
+        cfg = yaml.dump(
+            {'metric': [dict(name="test1",
+                        event_type="test.*",
+                        type="delta",
+                        unit="B",
+                        volume="$.payload.volume",
+                        resource_id="$.payload.resource_id",
+                        project_id="$.payload.project_id",
+                        user_metadata="$.payload.metadata",
+                        metadata={'proj': '$.payload.project_id'})]})
+        self._load_meter_def_file(cfg)
+        c = list(self.handler.build_sample(USER_META))
+        self.assertEqual(1, len(c))
+        s1 = c[0].as_dict()
+        meta = {'user_metadata': {'xyz': 'abc'}, 'proj': s1['project_id']}
+        self.assertEqual(meta, s1['resource_metadata'])
+
     def test_multi_match_event_meter(self):
         cfg = yaml.dump(
             {'metric': [dict(name="test1",
@@ -510,7 +554,7 @@ class TestMeterProcessing(test.BaseTestCase):
                              resource_id="$.payload.resource_id",
                              project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(NOTIFICATION))
+        c = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(2, len(c))
 
     def test_multi_meter_payload(self):
@@ -524,7 +568,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         project_id="$.payload.initiator.project_id",
                         lookup=["name", "volume", "unit"])]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(MIDDLEWARE_EVENT))
+        c = list(self.handler.build_sample(MIDDLEWARE_EVENT))
         self.assertEqual(2, len(c))
         s1 = c[0].as_dict()
         self.assertEqual('storage.objects.outgoing.bytes', s1['name'])
@@ -548,7 +592,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         project_id="$.payload.initiator.project_id",
                         lookup=["name", "unit"])]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(event))
+        c = list(self.handler.build_sample(event))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         self.assertEqual('storage.objects.outgoing.bytes', s1['name'])
@@ -568,7 +612,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         project_id="$.payload.initiator.project_id",
                         lookup="name")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(event))
+        c = list(self.handler.build_sample(event))
         self.assertEqual(0, len(c))
 
     def test_multi_meter_payload_all_multi(self):
@@ -584,7 +628,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         lookup=['name', 'type', 'unit', 'volume',
                                 'resource_id', 'project_id', 'user_id'])]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(FULL_MULTI_MSG))
+        c = list(self.handler.build_sample(FULL_MULTI_MSG))
         self.assertEqual(2, len(c))
         msg = FULL_MULTI_MSG['payload']
         for idx, val in enumerate(c):
@@ -612,7 +656,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         project_id="$.payload.initiator.project_id",
                         lookup=["name", "unit", "volume"])]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(event))
+        c = list(self.handler.build_sample(event))
         self.assertEqual(0, len(c))
         LOG.warning.assert_called_with('Only 0 fetched meters contain '
                                        '"volume" field instead of 2.')
@@ -631,7 +675,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         project_id="$.payload.initiator.project_id",
                         lookup=["name", "unit", "volume"])]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(event))
+        c = list(self.handler.build_sample(event))
         self.assertEqual(0, len(c))
         LOG.warning.assert_called_with('Only 1 fetched meters contain '
                                        '"volume" field instead of 2.')
@@ -648,7 +692,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         resource_id="$.payload.host + '_'"
                                     " + $.payload.nodename")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(METRICS_UPDATE))
+        c = list(self.handler.build_sample(METRICS_UPDATE))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         self.assertEqual('compute.node.cpu.percent', s1['name'])
@@ -667,7 +711,7 @@ class TestMeterProcessing(test.BaseTestCase):
                         resource_id="$.payload.host + '_'"
                                     " + $.payload.nodename")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(METRICS_UPDATE))
+        c = list(self.handler.build_sample(METRICS_UPDATE))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         self.assertEqual('compute.node.cpu.frequency', s1['name'])
@@ -685,7 +729,7 @@ class TestMeterProcessing(test.BaseTestCase):
                                ".value",
                         resource_id="'prefix-' + $.payload.nodename")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(METRICS_UPDATE))
+        c = list(self.handler.build_sample(METRICS_UPDATE))
         self.assertEqual(1, len(c))
         s1 = c[0].as_dict()
         self.assertEqual('compute.node.cpu.frequency', s1['name'])
@@ -710,5 +754,131 @@ class TestMeterProcessing(test.BaseTestCase):
                              resource_id="$.payload.resource_id",
                              project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
-        c = list(self.handler.process_notification(NOTIFICATION))
+        c = list(self.handler.build_sample(NOTIFICATION))
         self.assertEqual(1, len(c))
+
+    def test_multi_files_multi_meters(self):
+        cfg1 = yaml.dump(
+            {'metric': [dict(name="test1",
+                             event_type="test.create",
+                             type="delta",
+                             unit="B",
+                             volume="$.payload.volume",
+                             resource_id="$.payload.resource_id",
+                             project_id="$.payload.project_id")]})
+        cfg2 = yaml.dump(
+            {'metric': [dict(name="test2",
+                             event_type="test.create",
+                             type="delta",
+                             unit="B",
+                             volume="$.payload.volume",
+                             resource_id="$.payload.resource_id",
+                             project_id="$.payload.project_id")]})
+        self._load_meter_def_file([cfg1, cfg2])
+        data = list(self.handler.build_sample(NOTIFICATION))
+        self.assertEqual(2, len(data))
+        expected_names = ['test1', 'test2']
+        for s in data:
+            self.assertIn(s.as_dict()['name'], expected_names)
+
+    def test_multi_files_duplicate_meter(self):
+        cfg1 = yaml.dump(
+            {'metric': [dict(name="test",
+                             event_type="test.create",
+                             type="delta",
+                             unit="B",
+                             volume="$.payload.volume",
+                             resource_id="$.payload.resource_id",
+                             project_id="$.payload.project_id")]})
+        cfg2 = yaml.dump(
+            {'metric': [dict(name="test",
+                             event_type="test.create",
+                             type="delta",
+                             unit="B",
+                             volume="$.payload.volume",
+                             resource_id="$.payload.resource_id",
+                             project_id="$.payload.project_id")]})
+        self._load_meter_def_file([cfg1, cfg2])
+        data = list(self.handler.build_sample(NOTIFICATION))
+        self.assertEqual(1, len(data))
+        self.assertEqual(data[0].as_dict()['name'], 'test')
+
+    def test_multi_files_empty_payload(self):
+        event = copy.deepcopy(MIDDLEWARE_EVENT)
+        del event['payload']['measurements']
+        cfg1 = yaml.dump(
+            {'metric': [dict(name="$.payload.measurements.[*].metric.[*].name",
+                             event_type="objectstore.http.request",
+                             type="delta",
+                             unit="$.payload.measurements.[*].metric.[*].unit",
+                             volume="$.payload.measurements.[*].result",
+                             resource_id="$.payload.target_id",
+                             project_id="$.payload.initiator.project_id",
+                             lookup="name")]})
+        cfg2 = yaml.dump(
+            {'metric': [dict(name="$.payload.measurements.[*].metric.[*].name",
+                             event_type="objectstore.http.request",
+                             type="delta",
+                             unit="$.payload.measurements.[*].metric.[*].unit",
+                             volume="$.payload.measurements.[*].result",
+                             resource_id="$.payload.target_id",
+                             project_id="$.payload.initiator.project_id",
+                             lookup="name")]})
+        self._load_meter_def_file([cfg1, cfg2])
+        data = list(self.handler.build_sample(event))
+        self.assertEqual(0, len(data))
+
+    def test_multi_files_unmatched_meter(self):
+        cfg1 = yaml.dump(
+            {'metric': [dict(name="test1",
+                             event_type="test.create",
+                             type="delta",
+                             unit="B",
+                             volume="$.payload.volume",
+                             resource_id="$.payload.resource_id",
+                             project_id="$.payload.project_id")]})
+        cfg2 = yaml.dump(
+            {'metric': [dict(name="test2",
+                        event_type="test.update",
+                        type="delta",
+                        unit="B",
+                        volume="$.payload.volume",
+                        resource_id="$.payload.resource_id",
+                        project_id="$.payload.project_id")]})
+        self._load_meter_def_file([cfg1, cfg2])
+        data = list(self.handler.build_sample(NOTIFICATION))
+        self.assertEqual(1, len(data))
+        self.assertEqual(data[0].as_dict()['name'], 'test1')
+
+    @mock.patch('ceilometer.meter.notifications.LOG')
+    def test_multi_files_bad_meter(self, LOG):
+        cfg1 = yaml.dump(
+            {'metric': [dict(name="test1",
+                             event_type="test.create",
+                             type="delta",
+                             unit="B",
+                             volume="$.payload.volume",
+                             resource_id="$.payload.resource_id",
+                             project_id="$.payload.project_id"),
+                        dict(name="bad_test",
+                             type="bad_type",
+                             event_type="bar.create",
+                             unit="foo", volume="bar",
+                             resource_id="bea70e51c7340cb9d555b15cbfcaec23")]})
+        cfg2 = yaml.dump(
+            {'metric': [dict(name="test2",
+                             event_type="test.create",
+                             type="delta",
+                             unit="B",
+                             volume="$.payload.volume",
+                             resource_id="$.payload.resource_id",
+                             project_id="$.payload.project_id")]})
+        self._load_meter_def_file([cfg1, cfg2])
+        data = list(self.handler.build_sample(NOTIFICATION))
+        self.assertEqual(2, len(data))
+        expected_names = ['test1', 'test2']
+        for s in data:
+            self.assertIn(s.as_dict()['name'], expected_names)
+        args, kwargs = LOG.error.call_args_list[0]
+        self.assertEqual("Error loading meter definition: %s", args[0])
+        self.assertTrue(args[1].endswith("Invalid type bad_type specified"))
