@@ -13,65 +13,71 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 import abc
+from unittest import mock
 
-import mock
 from oslotest import base
-import six
-from six.moves.urllib import parse as url_parse
+from urllib import parse as urlparse
 
 from ceilometer.network.statistics.opendaylight import driver
 from ceilometer import service
 
 
-@six.add_metaclass(abc.ABCMeta)
-class _Base(base.BaseTestCase):
+class _Base(base.BaseTestCase, metaclass=abc.ABCMeta):
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def flow_data(self):
         pass
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def port_data(self):
         pass
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def table_data(self):
         pass
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def topology_data(self):
         pass
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def switch_data(self):
         pass
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def user_links_data(self):
         pass
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def active_hosts_data(self):
         pass
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def inactive_hosts_data(self):
         pass
 
-    fake_odl_url = url_parse.ParseResult('opendaylight',
-                                         'localhost:8080',
-                                         'controller/nb/v2',
-                                         None,
-                                         None,
-                                         None)
+    fake_odl_url = urlparse.ParseResult('opendaylight',
+                                        'localhost:8080',
+                                        'controller/nb/v2',
+                                        None,
+                                        None,
+                                        None)
 
-    fake_params = url_parse.parse_qs('user=admin&password=admin&scheme=http&'
-                                     'container_name=default&auth=basic')
+    fake_params = urlparse.parse_qs('user=admin&password=admin&scheme=http&'
+                                    'container_name=default&auth=basic')
 
     fake_params_multi_container = (
-        url_parse.parse_qs('user=admin&password=admin&scheme=http&'
-                           'container_name=first&container_name=second&'
-                           'auth=basic'))
+        urlparse.parse_qs('user=admin&password=admin&scheme=http&'
+                          'container_name=first&container_name=second&'
+                          'auth=basic'))
 
     def setUp(self):
         super(_Base, self).setUp()
