@@ -18,14 +18,17 @@
 """Tests for ceilometer agent manager"""
 import copy
 import datetime
-import fixtures
-import mock
+from unittest import mock
 
+import fixtures
 from keystoneauth1 import exceptions as ka_exceptions
 from stevedore import extension
 
 from ceilometer.compute import discovery as nova_discover
-from ceilometer.hardware import discovery
+from ceilometer.polling.dynamic_pollster import DynamicPollster
+from ceilometer.polling.dynamic_pollster import \
+    NonOpenStackApisPollsterDefinition
+from ceilometer.polling.dynamic_pollster import SingleMetricPollsterDefinitions
 from ceilometer.polling import manager
 from ceilometer.polling import plugin_base
 from ceilometer import sample
@@ -685,48 +688,6 @@ class TestPollingAgent(BaseAgent):
         self.assertFalse(self.PollsterKeystone.samples)
         self.assertFalse(self.notified_samples)
 
-    @mock.patch('ceilometer.polling.manager.LOG')
-    @mock.patch('ceilometer.nova_client.LOG')
-    def test_hardware_discover_fail_minimize_logs(self, novalog, baselog):
-        class PollsterHardware(TestPollster):
-            discovery = 'tripleo_overcloud_nodes'
-
-        class PollsterHardwareAnother(TestPollster):
-            discovery = 'tripleo_overcloud_nodes'
-
-        self.mgr.extensions.extend([
-            extension.Extension('testhardware',
-                                None,
-                                None,
-                                PollsterHardware(self.CONF), ),
-            extension.Extension('testhardware2',
-                                None,
-                                None,
-                                PollsterHardwareAnother(self.CONF), )
-        ])
-        ext = extension.Extension('tripleo_overcloud_nodes',
-                                  None,
-                                  None,
-                                  discovery.NodesDiscoveryTripleO(self.CONF))
-        self.mgr.discoveries = (extension.ExtensionManager
-                                .make_test_instance([ext]))
-
-        poll_cfg = {
-            'sources': [{
-                'name': "test_hardware",
-                'interval': 10,
-                'meters': ['testhardware', 'testhardware2'],
-                'sinks': ['test_sink']}],
-            'sinks': [{
-                'name': 'test_sink',
-                'publishers': ["test"]}]
-        }
-        self.setup_polling(poll_cfg)
-        polling_tasks = self.mgr.setup_polling_tasks()
-        self.mgr.interval_task(list(polling_tasks.values())[0])
-        self.assertEqual(1, novalog.exception.call_count)
-        self.assertFalse(baselog.exception.called)
-
     @mock.patch('ceilometer.polling.manager.LOG')
     def test_polling_exception(self, LOG):
         source_name = 'test_pollingexception'
@@ -890,3 +851,24 @@ class TestPollingAgentPartitioned(BaseAgent):
             mock.call('static_3'),
             mock.call('static_4'),
         ], any_order=True)
+
+    def test_instantiate_dynamic_pollster_standard_pollster(self):
+        pollster_definition_only_required_fields = {
+            'name': "test-pollster", 'sample_type': "gauge", 'unit': "test",
+            'value_attribute': "volume", 'endpoint_type': "test",
+            'url_path': "v1/test/endpoint/fake"}
+        pollster = DynamicPollster(pollster_definition_only_required_fields)
+
+        self.assertIsInstance(pollster.definitions,
+                              SingleMetricPollsterDefinitions)
+
+    def test_instantiate_dynamic_pollster_non_openstack_api(self):
+        pollster_definition_only_required_fields = {
+            'name': "test-pollster", 'sample_type': "gauge", 'unit': "test",
+            'value_attribute': "volume",
+            'url_path': "v1/test/endpoint/fake", 'module': "module-name",
+            'authentication_object': "authentication_object"}
+        pollster = DynamicPollster(pollster_definition_only_required_fields)
+
+        self.assertIsInstance(pollster.definitions,
+                              NonOpenStackApisPollsterDefinition)
